#!/usr/bin/env python3

import pkg_resources

import click

@click.command()
def cmd_version():
    print(pkg_resources.get_distribution("habu").version)


if __name__ == '__main__':
    cmd_version()



#!/usr/bin/env python3

import click
import json
import logging

@click.command()
@click.option('-j', 'json_output', is_flag=True, default=False, help='Output in JSON format')
@click.argument('infile', type=click.File('r'), default='-')
def cmd_h1_scope(infile, json_output):
    """Parse HackerOne scope specification in BurpSuite JSON format

    Example:

    \b
    $ habu.h1.scope starbucks.json
    app.starbucks.com
    card.starbucks.com.sg
    cart.starbucks.co.jp
    ec.starbucks.com.cn
    gift.starbucks.co.jp
    login.starbucks.co.jp
    preview.starbucks.com
    www.istarbucks.co.kr
    www.starbucks.ca
    www.starbucks.co.jp
    www.starbucks.co.uk
    www.starbucks.com
    www.starbucks.com.br
    www.starbucks.com.cn
    www.starbucks.com.sg
    www.starbucks.de
    www.starbucks.fr
    www.starbucksreserve.com
    www.teavana.com
    """

    try:
        data = json.loads(infile.read())
    except Exception:
        logging.error('Can\'t parse json file')
        return False

    domains = set()

    for item in data['target']['scope']['include']:
        domains.add(item['host'].strip('^').strip('$').replace('\\', ''))

    if json_output:
        print(json.dumps(sorted(domains), indent=4))
    else:
        print('\n'.join(sorted(domains)))


if __name__ == '__main__':
    cmd_h1_scope()



#!/usr/bin/env python3

import json
import warnings

import click
import whois


def remove_duplicates(data):
    for key in data.keys():
        if isinstance(data[key], list):
            if hasattr(data[key][0], 'lower'):
                data[key] = list(set([ item.lower() for item in data[key] ]))
            else:
                data[key] = data[key][0]

    if isinstance(data['domain_name'], list):
        data['domain_name'] = data['domain_name'][0]

    return data


@click.command()
@click.argument('domain')
@click.option('--json', 'json_output', is_flag=True, default=False, help='Print the output in JSON format')
@click.option('--csv', 'csv_output', is_flag=True, default=False, help='Print the output in CSV format')
def cmd_whois_domain(domain, json_output, csv_output):
    """Simple whois client to check domain names.

    Example:

    \b
    $ habu.whois.domain google.com
    registrar                MarkMonitor, Inc.
    whois_server             whois.markmonitor.com
    creation_date            1997-09-15 04:00:00
    expiration_date          2028-09-14 04:00:00
    name_servers             ns1.google.com, ns2.google.com, ns3.google.com, ns4.google.com
    emails                   abusecomplaints@markmonitor.com, whoisrequest@markmonitor.com
    dnssec                   unsigned
    org                      Google LLC
    country                  US
    state                    CA
    """

    default_fields = [
        'registrar',
        'whois_server',
        'update_date',
        'creation_date',
        'expiration_date',
        'name_servers',
        'emails',
        'dnssec',
        'org',
        'country',
        'state',
        'city',
        'address',
    ]


    warnings.filterwarnings("ignore")

    data = whois.whois(domain)
    data = remove_duplicates(data)

    if json_output:
        print(json.dumps(data, indent=4, default=str))
        return True

    for field in default_fields:
        value = data.get(field, None)
        if not value:
            continue

        if csv_output:
            if not isinstance(value, list):
                value = [value]
            for v in sorted(value):
                print('"{}","whois.{}","{}"'.format(domain, field, v))
        else:
            if isinstance(value, list):
                value = ', '.join(sorted(value))
            print('{:<25}{}'.format(field, value))


if __name__ == '__main__':
    cmd_whois_domain()



#!/usr/bin/env python3

import ipaddress
import socket
import ssl

import click
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.x509 import NameOID


def cert_get_names(cert_data):

    try:
        cert = x509.load_pem_x509_certificate(cert_data, default_backend())
    except ValueError:
        pass

    try:
        cert = x509.load_der_x509_certificate(cert_data, default_backend())
    except ValueError:
        raise ValueError("No recognized cert format. Allowed: PEM or DER")

    names = set()
    names.add(cert.subject.get_attributes_for_oid(NameOID.COMMON_NAME)[0].value.lower())

    try:
        alt_names = cert.extensions.get_extension_for_class(x509.SubjectAlternativeName)
    except x509.extensions.ExtensionNotFound:
        alt_names = None

    if alt_names:
        for alt_name in alt_names.value.get_values_for_type(x509.DNSName):
            names.add(alt_name.lower())

    return list(sorted(names))


@click.command()
@click.option('-p', 'ports', default='443', help='Ports to connect to (comma separated list)')
@click.option('-i', 'infile', type=click.File('r'), default='-', help='Input file (Default: stdin)')
@click.option('-t', 'timeout', type=click.FLOAT, default=1, help='Time to wait for each connection')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.argument('network', default=None, required=False)
def cmd_cert_names(infile, ports, timeout, verbose, network):
    """
    Connects to each host/port and shows a summary of the certificate names.

    The hosts to connect to are taken from two possible options:

    1) -i option (stdin by default). A file where each line is a host or network
    2) An argument that can be a host or network

    If you use both methods, the hosts and networks are merged into one list.

    Example:

    \b
    $ habu.cert.names 2.18.60.240/29
    2.18.60.241         443 i.s-microsoft.com microsoft.com privacy.microsoft.com
    2.18.60.242         443 aod-ssl.itunes.apple.com aod.itunes.apple.com aodp-ssl.itunes.apple.com
    2.18.60.243         443 *.mlb.com mlb.com
    2.18.60.244         443 [SSL: TLSV1_ALERT_INTERNAL_ERROR] tlsv1 alert internal error (_ssl.c:1056)
    2.18.60.245         443 cert2-cn-public-ubiservices.ubi.com cert2-cn-public-ws-ubiservices.ubi.com
    2.18.60.246         443 *.blog.sina.com.cn *.dmp.sina.cn

    aod.itunes.apple.com
    aodp-ssl.itunes.apple.com
    aod-ssl.itunes.apple.com
    *.blog.sina.com.cn
    cert2-cn-public-ubiservices.ubi.com
    cert2-cn-public-ws-ubiservices.ubi.com
    *.dmp.sina.cn
    i.s-microsoft.com microsoft.com
    *.mlb.com mlb.com
    privacy.microsoft.com
    """

    ports = [ int(port) for port in ports.split(',') if int(port) > 0 and int(port) <= 65535 ]

    hosts = set()

    if network:
        try:
            network = ipaddress.ip_network(network, strict=False)
        except Exception as e:
            click.echo(e, err=True)
            return False

        if len(list(network.hosts())) == 0:
            hosts |= set([ipaddress.ip_address(str(network).split('/')[0])])
        else:
            hosts |= { host for host in network.hosts() }

    if not infile.isatty():
        for network in infile.read().split('\n'):
            if network:
                try:
                    network = ipaddress.ip_network(network, strict=False)
                    hosts |= { host for host in network.hosts() }
                except Exception:
                    click.echo('Ignoring invalid host/network: {}'.format(network), err=True)
                    continue

    hosts = sorted(hosts)
    all_names = set()

    for host in hosts:
        for port in ports:
            print('{:<16} {:>6}'.format(str(host), port), end=' ')

            context = ssl.create_default_context()
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE

            try:
                with socket.create_connection((str(host), int(port)), timeout=timeout) as sock:
                    with context.wrap_socket(sock) as ssock:
                        cert = ssock.getpeercert(binary_form=True)
                        names = cert_get_names(cert)
                        all_names |= set(names)
                        print(' '.join(names))
            except Exception as e:
                print(e)

    print('')
    print('\n'.join(sorted(all_names)))


if __name__ == '__main__':
    cmd_cert_names()



import ipaddress

from habu.lib.identify import identify
from habu.lib.extract import guess_item_type

from habu.lib.ip2asn import ip2asn

def expand_ip(addr):

    result = {}

    ip = ipaddress.ip_network(addr, strict=False)

    prop_host = [ 'version', 'is_multicast', 'is_global', 'is_unspecified', 'is_reserved', 'is_loopback', 'is_link_local' ]

    prop_network = [ 'prefixlen', 'netmask', 'network_address', 'broadcast_address', 'num_addresses' ]

    for prop in prop_host:
        result[prop] = getattr(ip, prop)

    if ip.num_addresses > 1:
        for prop in prop_network:
            result[prop] = getattr(ip, prop)

    return result


expanders = {}
expanders['ipv4_address'] = []
expanders['ipv4_network'] = []
expanders['ipv6_address'] = []
expanders['ipv6_network'] = []
expanders['unknown'] = []
#expanders['IPAddress'].append(ip2asn)
expanders['ipv4_address'].append(expand_ip)
expanders['ipv4_network'].append(expand_ip)
expanders['ipv6_address'].append(expand_ip)
expanders['ipv6_network'].append(expand_ip)



def expand(item):

    family = guess_item_type(item)

    result = {
        'item': item,
        'family' : family,
    }

    for expander in expanders[family]:
        result.update(expander(item))

    return result





#!/usr/bin/env python3

import ipaddress
import json
import logging
import os
import os.path
import socket
import ssl
import sys
from pathlib import Path
from time import sleep

import dns.resolver
import dns.zone
import requests
import requests_cache
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from cryptography.x509 import NameOID
import tldextract

from habu.lib.shodan import shodan_get_result
from habu.lib.tomorrow3 import threads
from habu.lib import libdns
from habu.lib.loadcfg import loadcfg
from habu.lib.web_links import web_links


config = loadcfg()

def fqdns_from_ct_log(domain, cache=True, verbose=False):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if cache:
        homedir = Path(os.path.expanduser('~'))
        requests_cache.install_cache(str((homedir / '.habu_requests_cache')), expire_after=3600)

    fqdns = set()

    if verbose:
        print("Downloading subdomain list from https://crt.sh ...", file=sys.stderr)

    req = requests.get("https://crt.sh/?q=%.{d}&output=json".format(d=domain))

    if req.status_code != 200:
        print("[X] Information not available!")
        return False

    json_data = json.loads(req.text)

    for data in json_data:
        name = data['name_value'].lower()
        if '*' not in name:
            fqdns.add(name)

    return fqdns


def fqdns_from_certificate(cert_data):

    try:
        cert = x509.load_pem_x509_certificate(cert_data, default_backend())
    except ValueError:
        pass

    try:
        cert = x509.load_der_x509_certificate(cert_data, default_backend())
    except ValueError:
        raise ValueError("No recognized cert format. Allowed: PEM or DER")

    names = set()
    names.add(cert.subject.get_attributes_for_oid(NameOID.COMMON_NAME)[0].value.lower().rstrip('.'))

    try:
        alt_names = cert.extensions.get_extension_for_class(x509.SubjectAlternativeName)
    except x509.extensions.ExtensionNotFound:
        alt_names = None

    if alt_names:
        for alt_name in alt_names.value.get_values_for_type(x509.DNSName):
            names.add(alt_name.lower().rstrip('.'))

    return list(sorted(names))



def fqdns_from_hosts(hosts, domains, timeout=2):

    result = set()

    addrs = {}

    for host in list(hosts):
        resolves_to = libdns.resolve(host)
        if resolves_to:
            for addr in resolves_to:
                addrs[addr] = set(addrs.get(addr, [])) | set(host)

    for addr, fqdns in addrs.items():

        shodan_result = shodan_get_result(addr)

        if not shodan_result:
            continue

        for port in shodan_result['data']:

            if port['transport'] != 'tcp':
                continue

            if 'ssl' not in port:
                continue

            logging.info('connecting to {}:{}'.format(addr, port['port']))

            for fqdn in fqdns:

                context = ssl.create_default_context()
                context.check_hostname = False
                context.verify_mode = ssl.CERT_NONE

                try:
                    with socket.create_connection((str(addr), int(port['port'])), timeout=timeout) as sock:
                        with context.wrap_socket(sock) as ssock:
                            cert = ssock.getpeercert(binary_form=True)
                            fqdns_in_cert = set(fqdns_from_certificate(cert))
                            logging.debug('obtained fqdns: {}'.format(', '.join(fqdns_in_cert)))
                            result |= fqdns_in_cert
                except Exception as e:
                    break

    # Remove FQDNs that are not in any of the domains
    for fqdn in list(result):
        if not any([fqdn.endswith(domain) for domain in domains]):
            result.remove(fqdn)

    return result



@threads(20)
def dns_query(hostname):
    try:
        answer = dns.resolver.query(hostname)
        return answer
    except Exception:
        return None


def fqdns_from_brute_force(domain):

    fqdns = set()
    nxdomain = None

    try:
        nxdomain = dns.resolver.query('nonexistent456789.' + domain)
    except dns.resolver.NXDOMAIN:
        nxdomain = None

    with (config['DATADIR'] / 'subdomains.txt').open() as subdomains_file:
        subdomains_txt = subdomains_file.read()

    subdomains = { s + '.' + domain for s in subdomains_txt.split('\n') }

    subdomains.add(domain)

    jobs = [dns_query(sd) for sd in subdomains ]

    while True:
        if any([ job.running() for job in jobs]):
            sleep(1)
        else:
            break

    for job in jobs:

        result = job.result()

        if not hasattr(result, 'rrset'):
            continue

        if nxdomain and nxdomain.rrset[0] == result.rrset[0]:
            continue

        if str(result.rrset[0]).startswith('127.'):
            continue

        for r in result.rrset:
            fqdns.add(str(result.qname).rstrip('.'))

    return fqdns


def fqdns_from_weblinks(hostname, domains, timeout=5):

    fqdns = set()

    addrs = libdns.resolve(hostname)

    if not addrs:
        return fqdns

    addr = addrs[0]

    shodan_result = shodan_get_result(addr)

    if not shodan_result:
        return fqdns

    for port in shodan_result['data']:

        if 'http' not in port:
            continue

        if 'ssl' in port:
            scheme = 'https'
        else:
            scheme = 'http'

        url = '{}://{}:{}'.format(scheme, hostname, port['port'])

        logging.info('connecting to {}'.format(url))

        links = web_links(url)

        for link in links:
            tld = tldextract.extract(link)
            domain = '{}.{}'.format(tld.domain, tld.suffix)
            if domain in domains:
                if tld.subdomain:
                    fqdns.add('{}.{}.{}'.format(tld.subdomain, tld.domain, tld.suffix))

    return fqdns





#!/usr/bin/env python3

import ipaddress
import json
import logging

import click


def operator_gt(item, field, value):
    return field in item and item[field] > value

def operator_ge(item, field, value):
    return field in item and item[field] >= value

def operator_lt(item, field, value):
    return field in item and item[field] < value

def operator_le(item, field, value):
    return field in item and item[field] >= value

def operator_eq(item, field, value):
    return field in item and item[field] == value

def operator_ne(item, field, value):
    return field in item and item[field] != value

def operator_in_network(value1, value2):
    net1 = ipaddress.ip_network(value1)
    net2 = ipaddress.ip_network(value2)
    return net1.prefixlen >= net2.prefixlen and net2.overlaps(net1)

def operator_contains_network(value1, value2):
    net1 = ipaddress.ip_network(value1)
    net2 = ipaddress.ip_network(value2)
    return net1.prefixlen <= net2.prefixlen and net1.overlaps(net2)

def operator_in(item, field, value):

    if field not in item:
        return False

    try:
        ipaddress.ip_network(item[field])
        ipaddress.ip_network(value)
        return operator_in_network(item[field], value)
    except ValueError:
        pass

    values = value.split(',')
    return field in item and item[field] in values

def operator_contains(item, field, value):

    if field not in item:
        return False

    try:
        ipaddress.ip_network(item[field])
        ipaddress.ip_network(value)
        return operator_contains_network(item[field], value)
    except ValueError:
        pass

    return field in item and value in item[field]

def operator_defined(item, field, value):
    return field in item

def operator_undefined(item, field, value):
    return field not in item

def operator_true(item, field, value):
    return field in item and item[field] is True

def operator_false(item, field, value):
    return field in item and item[field] is False


operators = {
    'gt': operator_gt,
    'lt': operator_lt,
    'eq': operator_eq,
    'ne': operator_ne,
    'ge': operator_ge,
    'le': operator_le,
    'in': operator_in,
    'contains': operator_contains,
    'defined': operator_defined,
    'undefined': operator_undefined,
    'true': operator_true,
    'false': operator_false,
}

def operate(item, field, operator, value):
    return operators[operator](item, field, value)


@click.command()
@click.option('-i', 'infile', type=click.File('r'), default='-', help='Input file (Default: stdin)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('--not', 'negated', is_flag=True, default=False, help='Negate the comparison')
@click.argument('field', type=click.STRING)
@click.argument('operator', type=click.Choice(operators.keys()))
@click.argument('value', default=None, required=False)
def cmd_data_filter(infile, verbose, negated, field, operator, value):
    """Filter data based on operators.

    Example:

    \b
    $ cat /var/log/auth.log | habu.data.extract.ipv4 | habu.data.enrich | habu.data.filter cc eq US
    [
        {
            "item": "8.8.8.8",
            "family": "ipv4_address",
            "asn": "15169",
            "net": "8.8.8.0/24",
            "cc": "US",
            "rir": "ARIN",
            "asname": "GOOGLE - Google LLC, US"
        }
    ]
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if value is None and operator not in ['defined', 'undefined', 'true', 'false']:
        click.echo('Operator {} requires a value'.format(operator), err=True)

    try:
        data = json.loads(infile.read())
    except ValueError as e:
        print(e)
        click.echo('Invalid input data. Whe expect JSON here.', err=True)
        return False

    result = []

    for item in data:
        if negated:
            if not operate(item, field, operator, value):
                result.append(item)
        else:
            if operate(item, field, operator, value):
                result.append(item)

    print(json.dumps(result, indent=4))


if __name__ == '__main__':
    cmd_data_filter()



import os
from pathlib import Path

import requests
import requests_cache
from bs4 import BeautifulSoup


def get_vhosts(ip, first=1, no_cache=False):
    """Returns a list of webs hosted on IP (checks bing.com)
    >>> 'www.bing.com' in vhosts(204.79.197.200)
    True
    """

    if not no_cache:
        homedir = Path(os.path.expanduser('~'))
        requests_cache.install_cache(str(homedir / '.habu_requests_cache'), expire_after=3600)

    url = "http://www.bing.com/search?q=ip:{ip} &first={first}".format(ip=ip, first=first)

    response = requests.get(url)

    soup = BeautifulSoup(response.text, "html.parser")

    vhosts = set()

    for h2 in soup.find_all('h2'):
        for link in h2.find_all('a'):
            href = link.get('href')

            if href.startswith('http://') or href.startswith('https://'):
                vhost = href.split('/')[2]
                vhosts.add(vhost)

    return list(vhosts)



import regex
import validators


def check_luhn(cc):

    sum = 0
    parity = len(cc) % 2

    for i, digit in enumerate([int(x) for x in cc]):
        if i % 2 == parity:
            digit *= 2
            if digit > 9:
                digit -= 9
        sum += digit

    return sum % 10 == 0


def credit_card(text):

    rx = r'[0-9]{13,19}'
    m = regex.findall(rx, text, overlapped=True)
    #print('Before luhn', len(m))
    valid = []
    for possible in m:
        if check_luhn(possible):
            valid.append(possible)
    #print('After luhn', len(valid))
    return valid



validator_functions = {
    'ipv4_address' : validators.ipv4,
    'ipv4_network' : validators.ipv4_cidr,
    'ipv6_address' : validators.ipv6,
    'ipv6_network' : validators.ipv6_cidr,
    'domain' : validators.domain,
    'email' : validators.email,
    'url' : validators.url,
    'mac_address' : validators.mac_address,
    'md5' : validators.md5,
    'sha1' : validators.sha1,
    'sha224' : validators.sha224,
    'sha256' : validators.sha256,
    'sha512' : validators.sha512,
    'pan' : credit_card,
}


def guess_item_type(item):
    for item_type, function in validator_functions.items():
        if function(item):
            return item_type
    return None


if __name__ == '__main__':

    text = """
    VISA:
    4024007141679679
    4929838390718057
    4556789306248228608
    MasterCard:
    5425016886151184
    2221005662102894
    5535224890915019
    American Express (AMEX):
    341200896841032
    349764688249508
    372421022345415
    Discover:
    6011793427332832
    6011260018752893
    6011679156694765000
    JCB:
    3528351232225189
    3544543527236778
    3533678501612139597
    Diners Club - North America:
    5456878950482775
    5524961995313599
    5547678103880250
    Diners Club - Carte Blanche:
    30225940317115
    30473596040272
    30272074894170
    Diners Club - International:
    36682688303216
    36705742855069
    36622473207743
    Maestro:
    6759336225624384
    6763077648725655
    5020380847601144
    Visa Electron:
    4508018645020067
    4913484711396667
    4913987879409494
    InstaPayment:
    6390379018975931
    6380946373456228
    6387276287896233
    """

    #print(credit_card(text))

    test_items = [
        '8.8.4.4',
        'www.google.com',
        'google.com',
        'https://www.google.com',
        'xxss://www.google.com',
        '60b725f10c9c85c70d97880dfe8191b3',
        '3f786850e387550fdab836ed7e6dc881de23001b',
        '7c297c1793fdad2ac52a68bdd6b8fde3eb59b99c3f8c44710fde5fd7',
        '87428fc522803d31065e7bce3cf03fe475096631e5e07bbd7a0fde60c4cf25c7',
        '162b0b32f02482d5aca0a7c93dd03ceac3acd7e410a5f18f3fb990fc958ae0df6f32233b91831eaf99ca581a8c4ddf9c8ba315ac482db6d4ea01cc7884a635be',
        'test@asd.com',
        '@æ|nothingforhere!',
        '4913484711396667',
    ]

    for item in test_items:
        print(item, guess_item_type(item))



#!/usr/bin/env python3

import ipaddress
import json
import logging
import os
import sys

import click

from habu.lib import libdns

from habu.lib.loadcfg import loadcfg
from habu.lib.shodan import shodan_get_result

@click.command()
@click.argument('ip')
@click.option('--cache/--no-cache', 'cache', default=True)
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('--format', 'output_format', type=click.Choice(['txt', 'csv','json', 'nmap']), default='txt', help='Output format')
def cmd_shodan(ip, cache, verbose, output_format):
    """Simple shodan API client.

    Prints the JSON result of a shodan query.

    Example:

    \b
    $ habu.shodan 216.58.222.36
    asn                      AS15169
    isp                      Google
    hostnames                eze04s06-in-f4.1e100.net, gru09s17-in-f36.1e100.net
    country_code             US
    region_code              CA
    city                     Mountain View
    org                      Google
    open_ports               tcp/443, tcp/80
    """

    try:
        ipaddress.ip_address(ip)
    except ValueError:
        ip = libdns.resolve(ip)

    if ip and isinstance(ip, list):
        ip = ip[0]
        logging.info('Resolved to {}'.format(ip))

    if not ip:
        logging.error('Invalid IP address or unresolvable name')
        return False

    habucfg = loadcfg()

    if 'SHODAN_APIKEY' not in habucfg:
        print('You must provide a shodan apikey. Use the ~/.habu.json file (variable SHODAN_APIKEY), or export the variable HABU_SHODAN_APIKEY')
        print('Get your API key from https://www.shodan.io/')
        sys.exit(1)

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = shodan_get_result(ip, habucfg['SHODAN_APIKEY'], cache=cache, verbose=verbose)

    if output_format == 'json':
        print(json.dumps(data, indent=4))
        return True

    if not data:
        logging.error('Shodan seems to have no data for this host')
        return False

    if output_format == 'nmap':
        ports_string = ','.join(['{}:{}'.format(port['transport'][0].upper(), port['port']) for port in data['data']])
        print(ports_string, end='')
        return True

    default_fields = [
        'asn',
        'isp',
        'hostnames',
        'country_code',
        'region_code',
        'city',
        'org',
        'os',
    ]

    for field in default_fields:
        value = data.get(field, None)
        if not value:
            continue

        if output_format == 'csv':
            if not isinstance(value, list):
                value = [value]
            for v in sorted(value):
                print('"{}","shodan.{}","{}"'.format(ip, field, v))
        else:
            if isinstance(value, list):
                value = ', '.join(sorted(value))
            print('{:<25}{}'.format(field, value))

    if output_format == 'txt':
        ports_string = ', '.join(['{}/{}'.format(port['transport'], port['port']) for port in data['data']])
        print('{:<25}{}'.format('open_ports', ports_string))
    else:
        for port in data['data']:
            print('"{}","shodan.open_port","{}/{}"'.format(ip, port['transport'], port['port']))


if __name__ == '__main__':
    cmd_shodan()



#!/usr/bin/env python3

import dns.resolver
import dns.zone


def resolve(name, server=None):
    """resolves the name"""

    my_resolver = dns.resolver.Resolver()

    if server:
        my_resolver.nameservers = [server]

    result = []

    try:
        answers = my_resolver.query(str(name), lifetime=3)
    except Exception as e:
        answers = []

    answers = [ str(answer) for answer in answers ]

    return answers




def ns(domain):
    """returns a list with the NS servers for domain"""

    result = []

    try:
        answers = dns.resolver.query(domain, 'NS')
    except Exception:
        answers = []

    #if not answers:
    #    answers = []

    for rdata in answers:
        result.append(str(rdata.target).lower())

    return result


def mx(domain):
    """returns a list with the MX servers for domain"""

    result = []

    try:
        answers = dns.resolver.query(domain, 'MX')
    except Exception:
        answers = []

    #if not answers:
    #    answers = []

    for rdata in answers:
        result.append(str(rdata.exchange).lower())

    return result


def axfr(domain):
    """ returns a list with the ns that allows zone transfers"""

    allowed = []

    for server in ns(domain):

        try:
            z = dns.zone.from_xfr(dns.query.xfr(server, domain))
            if z.nodes.keys():
                allowed.append(server)
        except Exception as e:
            pass

    return allowed


if __name__ == '__main__':
    print(ns('securetia.com'))
    print(mx('securetia.com'))
    print(axfr('securetia.com'))




#!/usr/bin/env python3
import json

import click

from habu.lib.ip import get_public_ipv4, get_public_ipv6


@click.command()
@click.option('-4', '--ipv4', 'ipv4', is_flag=True, default=False, help='Print your public IPv4 address (default)')
@click.option('-6', '--ipv6', 'ipv6', is_flag=True, default=False, help='Print your public IPv6 address')
@click.option('-j', '--json', 'json_output', is_flag=True, default=False, help='Print the output in JSON format')
def cmd_ip(ipv4, ipv6, json_output):
    """Get the public IP address of the connection from https://api.ipify.org.

    Example:

    \b
    $ habu.ip
    80.219.53.185
    """

    result = {}

    if not ipv4 and not ipv6:
        ipv4 = True

    if ipv4:
        result['ipv4_address'] = get_public_ipv4()

    if ipv6:
        result['ipv6_address'] = get_public_ipv6()

    if json_output:
        print(json.dumps(result, indent=4))
    else:
        print('\n'.join([ v for v in result.values() ]))


if __name__ == '__main__':
    cmd_ip()



import ipaddress
import dns


def is_ipaddress(item):
    try:
        ipaddress.ip_address(item)
        return True
    except Exception:
        return False


def is_ipnetwork(item):
    try:
        ipaddress.ip_network(item)
        return True
    except Exception:
        return False


def is_domain(item):
    dns.name.from_text(item)



# We need to do more work on this function, because can fail on a lot of circumstances
def item_type(item):

    if item.startswith('http://') or item.startswith('https://'):
        return ['web']

    if '@' in item:
        return ['email']

        pass
    try:
        ipaddress.ip_network(item)
        return ['network']
    except Exception:
        pass

    # Si tiene servidores NS asociados, es un dominio o subdominio


    if habu.lib.dns.ns(item):
        return ['domain']

    # Si resuelve por DNS, es un hostname
    if habu.lib.dns.resolve(item):
        return ['fqdn']

    # Si no, no sabemos qué es.
    return None



if __name__ == '__main__':

    test_items = [
        'asd@asd.com',
        'microsoft.com',
        'invalid!domain.com',
        '8.8.8.8',
        '2001:0db8:85a3:0000:0000:8a2e:0370:7334',
        '8.8.8.0/24',
        'www.microsoft.com',
    ]

    for item in test_items:
        print(item, item_type(item))





#!/usr/bin/env python3

import logging
import os
import os.path

import requests
import requests_cache

from pathlib import Path

from habu.lib.loadcfg import loadcfg

config = loadcfg()

def shodan_get_result(ip, api_key=None, cache=True, verbose=False):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if cache:
        homedir = Path(os.path.expanduser('~'))
        requests_cache.install_cache(str(homedir / '.habu_requests_cache'), expire_after=3600)

    if not api_key:
        api_key = config['SHODAN_APIKEY']

    url = 'https://api.shodan.io/shodan/host/{}?key={}'.format(ip, api_key)

    r = requests.get(url)

    if r.status_code not in [200, 404]:
        logging.error(str(r))
        return {}

    if r.status_code == 404:
        return {}

    data = r.json()

    return data


def shodan_query(query, api_key, cache=True, verbose=False):

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    if cache:
        homedir = Path(os.path.expanduser('~'))
        requests_cache.install_cache(str(homedir / '.habu_requests_cache'), expire_after=3600)

    url = 'https://api.shodan.io/shodan/host/search?key={}&query={}'.format(api_key, query)

    r = requests.get(url)

    if r.status_code not in [200, 404]:
        logging.error(str(r))
        return {}

    if r.status_code == 404:
        return {}

    data = r.json()

    return data



#!/usr/bin/env python3

import ipaddress
import json
import logging

import click


@click.command()
@click.option('-i', 'infile', type=click.File('r'), default='-', help='Input file (Default: stdin)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('--json', 'json_output', is_flag=True, default=False, help='JSON output')
@click.argument('field', type=click.STRING)
def cmd_select(infile, json_output, verbose, field):
    """Select a field from a JSON input.

    Example:

    \b
    $ cat /var/log/auth.log | habu.extract.ipv4 | habu.expand | habu.filter cc eq US | habu.select asset
    8.8.8.7
    8.8.8.8
    8.8.8.9
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    try:
        data = json.loads(infile.read())
    except ValueError as e:
        print(e)
        click.echo('Invalid input data. Whe expect JSON here.', err=True)
        return False

    result = []

    if not isinstance(data, list):
        data = [data]

    for item in data:
        if field in item:
            result.append(item[field])

    if json_output:
        print(json.dumps(result, indent=4))
        return True

    for r in result:
        if isinstance(r, list):
            print('\n'.join(sorted(r)))
        else:
            print(r)


if __name__ == '__main__':
    cmd_select()



#!/usr/bin/env python3

import json
import logging

import dns.query
import dns.resolver
import dns.zone

import click

from habu.lib import libdns
from habu.lib.fqdn_finder import fqdns_from_ct_log, fqdns_from_hosts, fqdns_from_brute_force, fqdns_from_weblinks


@click.command()
@click.option('-t', 'timeout', type=click.FLOAT, default=1, help='Time to wait for each connection')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
@click.option('--debug', 'debug', is_flag=True, default=False, help='Debug output')
@click.option('--connect/--no-connect', 'connect', default=True, help='Get from known FQDNs open ports SSL certificates')
@click.option('--brute/--no-brute', 'brute_force', default=True, help='Run DNS brute force against domains')
@click.option('--links/--no-links', 'weblinks', default=True, help='Extract FQDNs from web site links')
@click.option('--xfr/--no-xfr', 'xfr', default=True, help='Try to do a DNS zone transfer against domains')
@click.option('--ctlog/--no-ctlog', 'ctlog', default=True, help='Try to get FQDNs from Certificate Transparency Logs')
@click.option('--json', 'json_output', is_flag=True, default=False, help='Print the output in JSON format')
@click.argument('domains', nargs=-1)
def cmd_fqdn_finder(domains, brute_force, connect, xfr, ctlog, weblinks, timeout, verbose, debug, json_output):
    """
    Uses various techniques to obtain valid FQDNs for the specified domains.

    \b
    1. Check for Certificate Transparency Logs
    2. Connect to specified ports, obtain SSL certificates and get FQDNs from them

    Next versions will also do the following:

    \b
    3. DNS Brute Force for common names
    4. Try DNS Zone Transfer first

    The results are cleaned to remove FQDNs that does not resolve by DNS

    Example:

    \b
    $ habu.fqdn.finder educacionit.com
    barometrosalarial.educacionit.com
    blog.educacionit.com
    ci.educacionit.com
    educacionit.com
    intranet.educacionit.com
    lecdev.educacionit.com
    lecweb.educacionit.com
    mail.educacionit.com
    plantillas.educacionit.com
    www.educacionit.com
    """

    domains = set(domains)
    #print(domains)

    if debug:
        logging.basicConfig(level=logging.DEBUG)
    elif verbose:
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.WARNING)

    fqdns = set()

    if xfr:
        logging.info('Trying to get DNS Zones with a Zone Transfer..')
        for domain in list(domains):
            for ns_server in libdns.ns(domain):
                try:
                    z = dns.zone.from_xfr(dns.query.xfr(ns_server, domain, lifetime=5))
                except Exception as e:
                    z = None
                if z:
                    logging.info('Got DNS Zone with XFR for {} domain'.format(domain))
                    domains.remove(domain) # it's safe to remove the domains if zone available via XFR?
                    for name in z.nodes.keys():
                        fqdns.add(str(name) + '.' + domain)
                    break

    if ctlog:
        for domain in domains:
            logging.info('Getting FQDNs for {} from certificate transparency logs...'.format(domain))
            result = fqdns_from_ct_log(domain)
            logging.info('Got {} FQDNs'.format(len(result)))
            fqdns |= result

    logging.info('Removing FQDNs that does not resolve by DNS...')
    for fqdn in list(fqdns):
        if not libdns.resolve(fqdn):
            logging.info('Removing {} because does not resolves by DNS'.format(fqdn))
            fqdns.remove(fqdn)

    if connect:
        logging.info('Getting FQDNs from SSL certificates')
        result = fqdns_from_hosts(fqdns, domains=domains)
        logging.info('Got {} FQDNs'.format(len(result)))
        fqdns |= result

    if weblinks:
        logging.info('Getting FQDNs from web links')
        for fqdn in list(fqdns):
            result = fqdns_from_weblinks(fqdn, domains=domains)
            logging.info('Got {} FQDNs'.format(len(result)))
            fqdns |= result

    if brute_force:
        logging.info('Running DNS brute force...')
        for domain in domains:
            result = fqdns_from_brute_force(domain)
            logging.info('Got {} FQDNs'.format(len(result)))
            fqdns |= result

    if json_output:
        print(json.dumps(sorted(fqdns), indent=4))
    else:
        print('\n'.join(sorted(fqdns)))


if __name__ == '__main__':
    cmd_fqdn_finder()



#!/usr/bin/env python3

import logging

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from habu.lib.iface import search_iface
from scapy.all import ARP, Ether, conf, srp


@click.command()
@click.argument('ip')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_arp_ping(ip, iface, verbose):
    """
    Send ARP packets to check if a host it's alive in the local network.

    Example:

    \b
    # habu.arp.ping 192.168.0.1
    Ether / ARP is at a4:08:f5:19:17:a4 says 192.168.0.1 / Padding
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    conf.verb = False

    if iface:
        iface = search_iface(iface)
        if iface:
            conf.iface = iface['name']
        else:
            logging.error('Interface {} not found. Use habu.interfaces to show valid network interfaces'.format(iface))
            return False

    res, unans = srp(Ether(dst="ff:ff:ff:ff:ff:ff")/ARP(pdst=ip), timeout=2)

    for _, pkt in res:
        if verbose:
            print(pkt.show())
        else:
            print(pkt.summary())


if __name__ == '__main__':
    cmd_arp_ping()



import logging
from time import sleep

import click

from habu.lib.auth import *
from habu.lib.auth.base import BaseAuth, ReturnCode


supported_services = {}

for subclass in BaseAuth.__subclasses__():
    for service in subclass.services:
        supported_services[service] = subclass


@click.command()
@click.option('-u', 'username', default=None, help='Username to crack')
@click.option('-U', 'username_file', type=click.File(), default=None, help='File with usernames to crack')
@click.option('-p', 'password', default=None, help='Password to test')
@click.option('-P', 'password_file', type=click.File(), default=None, help='File with passwords to test')
@click.option('-s', 'sleep_time', type=click.FLOAT, default=0, help='Seconds to sleep between probes (default: 0)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose mode')
@click.argument('host')
@click.argument('port', type=click.INT)
@click.argument('service', type=click.Choice(sorted(supported_services)))
def cmd_crack_service(host, port, username, password, username_file, password_file, service, sleep_time, verbose):
    """
    """

    if verbose:
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.WARN)

    subclass = supported_services[service]

    usernames = []
    passwords = []

    if username:
        usernames.append(username)

    if password:
        passwords.append(password)

    if username_file:
        for line in username_file.read().split('\n'):
            usernames.append(line)

    if password_file:
        for line in password_file.read().split('\n'):
            passwords.append(line)

    if not usernames:
        logging.error('I don\'t have usernames to try.')
        return False

    if not passwords:
        logging.error('I don\'t have passwords to try.')
        return False

    logging.info('We will test {} usernames and {} passwords, total intents: {}'.format(len(usernames), len(passwords), len(usernames) * len(passwords)))

    for username in usernames:
        for password in passwords:

            logging.info('Trying {} {}'.format(username, password))

            result = subclass(username=username, password=password, port=port, service=service, address=host).login()

            if result == ReturnCode.AUTH_OK:
                print('cracked! Username: {}, Password: {}'.format(username, password))
                return True

            if result == ReturnCode.AUTH_FAILED:
                #logging.info('AUTH_FAILED')
                print('AUTH_FAILED')
                sleep(sleep_time)
                continue

            if result == ReturnCode.ACCT_BLOCKED:
                print('The account seems to be blocked, trying next account (if any)')
                break

            if result == ReturnCode.CONN_TIMEOUT:
                print('The connections has timed out, maybe the service is not listening on that port or we have been blocked, stopping')
                return False

            if result == ReturnCode.CONN_REFUSED:
                print('The connections was refused, maybe the service was gone or we have been blocked, stopping')
                return False

            if result == ReturnCode.TLS_ERROR:
                print('An SSL/TLS error was encountered. Please, check what happened and, maybe, run with the --skip-tls-errors flag')
                return False


    print('We finished without find valid credentials. Sorry.')



if __name__ == '__main__':
    cmd_crack_service()



#!/usr/bin/env python3

import json
import warnings

import click
from ipwhois import IPWhois


@click.command()
@click.argument('ip')
@click.option('--json', 'json_output', is_flag=True, default=False, help='Print the output in JSON format')
@click.option('--csv', 'csv_output', is_flag=True, default=False, help='Print the output in CSV format')
def cmd_whois_ip(ip, json_output, csv_output):
    """Simple whois client to check IP addresses (IPv4 and IPv6).

    Example:

    \b
    $ habu.whois.ip 8.8.4.4
    asn                      15169
    asn_registry             arin
    asn_cidr                 8.8.4.0/24
    asn_country_code         US
    asn_description          GOOGLE - Google LLC, US
    asn_date                 1992-12-01
    """

    warnings.filterwarnings("ignore")

    default_fields = [
        'asn',
        'asn_registry',
        'asn_cidr',
        'asn_country_code',
        'asn_description',
        'asn_date',
    ]

    obj = IPWhois(ip)
    data = obj.lookup_rdap()

    if json_output:
        print(json.dumps(data, indent=4))
        return True

    for field in default_fields:
        value = data.get(field, None)
        if not field:
            continue

        if csv_output:
            print('"{}","whois.{}","{}"'.format(ip, field, value))
        else:
            print('{:<25}{}'.format(field, value))


if __name__ == '__main__':
    cmd_whois_ip()



#!/usr/bin/env python3

import logging

import click

logging.getLogger("scapy.runtime").setLevel(logging.ERROR)

from habu.lib.iface import search_iface
from scapy.all import IP, TCP, RandShort, conf, send, sr1


@click.command()
@click.argument('ip')
@click.option('-p', 'port', default=80, help='Port to use (default: 80)')
@click.option('-c', 'count', default=5, help='How many packets to send/receive (default: 5)')
@click.option('-i', 'iface', default=None, help='Interface to use')
@click.option('-g', 'graph', is_flag=True, default=False, help='Graph (requires matplotlib)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_isn(ip, port, count, iface, graph, verbose):
    """Create TCP connections and print the TCP initial sequence
    numbers for each one.

    \b
    $ sudo habu.isn -c 5 www.portantier.com
    1962287220
    1800895007
    589617930
    3393793979
    469428558

    Note: You can get a graphical representation (needs the matplotlib package)
    using the '-g' option to better understand the randomness.
    """

    conf.verb = False

    if iface:
        iface = search_iface(iface)
        if iface:
            conf.iface = iface['name']
        else:
            logging.error('Interface {} not found. Use habu.interfaces to show valid network interfaces'.format(iface))
            return False

    isn_values = []

    for _ in range(count):
        pkt = IP(dst=ip)/TCP(sport=RandShort(), dport=port, flags="S")
        ans = sr1(pkt, timeout=0.5)
        if ans:
            send(IP(dst=ip)/TCP(sport=pkt[TCP].sport, dport=port, ack=ans[TCP].seq + 1, flags='A'))
            isn_values.append(ans[TCP].seq)
            if verbose:
                ans.show2()

    if graph:

        try:
            import matplotlib.pyplot as plt
        except ImportError:
            print("To graph support, install matplotlib")
            return 1

        plt.plot(range(len(isn_values)), isn_values, 'ro')
        plt.show()

    else:

        for v in isn_values:
            print(v)

    return True


if __name__ == '__main__':
    cmd_isn()



#!/usr/bin/env python3

import ipaddress
import json
import logging

import click
import regex as re


def extract_ipv4(data):

    #regexp = re.compile(r'\s?((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\s?', flags=re.MULTILINE)
    regexp = re.compile(r'[\s():{}\[\]]{1}((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)[\s():{}\[\]]{1}', flags=re.MULTILINE)

    regexp = re.compile(r'[^0-9]?(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})[^0-9]?', flags=re.MULTILINE)

    match = regexp.finditer(data)

    result = []

    for m in match:
        try:
            ipaddress.ip_address(m.group(1))
            result.append(m.group(1))
        except ValueError:
            continue
        #ip = m.group(0).strip(' ():{}[]')
        #ip = ip.strip()
        #if ip:
        #    result.append(ip)

    return result

@click.command()
@click.argument('infile', type=click.File('r'), default='-')
@click.option('-j', '--json', 'json_output', is_flag=True, default=False, help='JSON output')
@click.option('-u', '--unique', 'unique', is_flag=True, default=False, help='Remove duplicates')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_data_extract_ipv4(infile, json_output, unique, verbose):
    """Extract IPv4 addresses from a file or stdin.

    Example:

    \b
    $ cat /var/log/auth.log | habu.data.extract.ipv4
    172.217.162.4
    23.52.213.96
    190.210.43.70
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    data = infile.read()

    result = extract_ipv4(data)

    if unique:
        result = list(sorted(set(result)))

    if not json_output:
        print('\n'.join(result))
        return True

    result_dict = [ { 'ipv4_address' : ip } for ip in result ]
    print(json.dumps(result_dict, indent=4))

if __name__ == '__main__':
    cmd_data_extract_ipv4()



import json
import operator

def pwdlyze(passwords):

    prefixes = {}
    postfixes = {}
    lengths = {}
    symbols = {}
    numbers = {}

    for password in passwords:

        if not password:
            continue

        prefix = password[0]
        postfix = password[-1]
        length = len(password)

        if prefix not in prefixes:
            prefixes[prefix] = 0

        prefixes[prefix] += 1

        if postfix not in postfixes:
            postfixes[postfix] = 0

        postfixes[postfix] += 1

        if length not in lengths:
            lengths[length] = 0

        lengths[length] += 1

    return {
        'prefixes' : sorted(prefixes.items(), key=operator.itemgetter(1), reverse=True)[:20],
        'postfixes' : sorted(postfixes.items(), key=operator.itemgetter(1), reverse=True)[:20],
        'lengths' : sorted(lengths.items(), key=operator.itemgetter(1), reverse=True)[:20],
    }


if __name__ == '__main__':
    with open('/home/f/code/SecLists/Passwords/darkc0de.txt', encoding='utf-8', errors='ignore') as pwdfile:
        passwords = pwdfile.read().split('\n')

    print(json.dumps(pwdlyze(passwords), indent=4))




#!/usr/bin/env python3

import json
import logging

import click

from habu.lib.expand import expand


@click.command()
@click.option('-i', 'infile', type=click.File('r'), default='-', help='Input file (Default: stdin)')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_expand(infile, verbose):
    """Expand data to add interesting information.

    Example:

    \b
    $ cat /var/log/auth.log | habu.extract.ipv4 | habu.expand
    [
        {
            "asset": "8.8.8.8",
            "family": "IPAddress",
            "asn": "15169",
            "net": "8.8.8.0/24",
            "cc": "US",
            "rir": "ARIN",
            "asname": "GOOGLE - Google LLC, US"
        },
        {
            "asset": "8.8.4.4",
            "family": "IPAddress",
            "asn": "15169",
            "net": "8.8.4.0/24",
            "cc": "US",
            "rir": "ARIN",
            "asname": "GOOGLE - Google LLC, US"
        }
    ]
    """

    if verbose:
        logging.basicConfig(level=logging.INFO, format='%(message)s')

    lines = infile.read().split('\n')

    result = [ expand(line) for line in lines if line ]

    print(json.dumps(result, indent=4))


if __name__ == '__main__':
    cmd_expand()



import logging
import urllib.parse
from pathlib import Path

import regex as re
import requests
import requests_cache
import urllib3
from bs4 import BeautifulSoup

urllib3.disable_warnings()

def web_links(url, cache=True):
    """Returns all the links found in a web site"""

    if cache:
        requests_cache.install_cache(str(Path.home() / '.habu_requests_cache'), expire_after=3600)

    links = set()

    try:
        r = requests.get(url, timeout=5, verify=False)
    except Exception as e:
        return links

    soup = BeautifulSoup(r.content, 'lxml')

    http_url_regex = re.compile('http(s)?://', flags=re.IGNORECASE)

    for link in soup.findAll('a'):

        href = link.get('href')
        href = urllib.parse.urljoin(url, href)

        if not http_url_regex.match(href):
            continue

        links.add(href.rstrip('/'))


    for script in soup.findAll('script'):
        if 'src' not in script.attrs:
            continue

        file_url = urllib.parse.urljoin(url, script.attrs['src'])

        file_url = file_url.rstrip('/')
        links.add(file_url)


    for css in soup.findAll('link', attrs={'rel': 'stylesheet'}):

        file_url = css.attrs['href']
        file_url = urllib.parse.urljoin(url, file_url)

        file_url = file_url.rstrip('/')
        links.add(file_url)


    return links



import ipaddress

def identify(asset):

    try:
        ipaddress.ip_address(asset)
        return 'IPAddress'
    except Exception:
        pass
    try:
        ipaddress.ip_network(asset)
        return 'IPNetwork'
    except Exception:
        pass

    return 'Unknown'

    '''
    if asset.startswith('http://') or asset.startswith('https://'):
        return 'web'
    if '@' in asset:
        return 'email'
    # Si tiene servidores NS asociados, es un dominio o subdominio
    if libdns.ns(asset):
        return 'domain'

    # Si resuelve por DNS, es un hostname
    if libdns.resolve(asset):
        return 'hostname'

    # Si no, no sabemos qué es.
    return None
    '''




#!/usr/bin/env python3

import json

import click

from habu.lib.http_tech import http_tech


@click.command()
@click.argument('url')
@click.option('--cache/--no-cache', 'cache', default=True)
@click.option('--format', 'output_format', type=click.Choice(['txt', 'csv','json']), default='txt', help='Output format')
@click.option('-v', 'verbose', is_flag=True, default=False, help='Verbose output')
def cmd_http_tech(url, cache, output_format, verbose):
    """Uses Wappalyzer apps.json database to identify technologies used on a web application.

    Reference: https://github.com/AliasIO/Wappalyzer

    Note: This tool only sends one request. So, it's stealth and not suspicious.

    \b
    $ habu.web.tech https://woocomerce.com
    Google Tag Manager       unknown
    MySQL                    unknown
    Nginx                    unknown
    PHP                      unknown
    Prototype                unknown
    RequireJS                unknown
    WooCommerce              3.8.0
    WordPress                5.2.4
    Yoast SEO                10.0.1
    """

    response = http_tech(url, cache=cache, verbose=verbose)

    if output_format == 'json':
        print(json.dumps(response, indent=4))
        return True

    for tech, data in response.items():
        if 'version' not in data:
            data['version'] = 'unknown'

        if output_format == 'csv':
            print('"{}","{}","{}"'.format(url, tech, data['version']))
        else:
            print('{:<25}{}'.format(tech, data['version']))


if __name__ == '__main__':
    cmd_http_tech()



import click
import json
import operator

def pwdpolicy(passwords, minlength=0, need_lower=0, need_upper=0, need_number=0, need_special=0, need_families=0):

    valid = []

    for password in passwords:

        if not password:
            continue

        if len(password) < minlength:
            continue

        password = password.decode(errors='ignore')

        are_lower = 0
        are_upper = 0
        are_number = 0
        are_special = 0

        families = 0

        for ch in password:

            #print(str(ch))
            #ch = ch.decode()

            if ch.islower():
                are_lower += 1
                if are_lower == 1:
                    families += 1
                continue

            if ch.isupper():
                are_upper += 1
                if are_upper == 1:
                    families += 1
                continue

            if ch.isnumeric():
                are_number += 1
                if are_number == 1:
                    families += 1
                continue

            are_special += 1
            if are_special == 1:
                families += 1

            print(
                'lower:', are_lower,
                'upper:', are_upper,
                'number:', are_number,
                'special:', are_special,
                'families:', families
            )

        if need_families > families:
            continue

        if need_lower > are_lower:
            continue

        if need_upper > are_upper:
            continue

        if need_special > are_special:
            continue

        valid.append(password)

    return valid


@click.command()
@click.argument('infile', type=click.File('rb', errors='ignore'), default='-')
@click.option('--length', 'minlength', default=0)
@click.option('--number', 'need_number', default=0)
@click.option('--lower', 'need_lower', default=0)
@click.option('--upper', 'need_upper', default=0)
@click.option('--special', 'need_special', default=0)
@click.option('--families', 'need_families', default=0)
def cmd_pwd_policy(infile, minlength, need_number, need_lower, need_upper, need_special, need_families):

    passwords = infile.read().split(b'\n')

    valid = pwdpolicy(passwords, minlength=minlength, need_number=need_number, need_lower=need_lower, need_upper=need_upper, need_special=need_special, need_families=need_families)

    print('\n'.join(valid))

if __name__ == '__main__':
    #with open('/home/f/code/SecLists/Passwords/darkc0de.txt', encoding='utf-8', errors='ignore') as pwdfile:
    #    passwords = pwdfile.read().split('\n')

    #print(json.dumps(pwdlyze(passwords), indent=4))


    cmd_pwd_policy()


